#!/usr/bin/python3
# -*- coding=utf-8 -*-
r"""

"""
import os.path as p
from ..exceptions import ConfigNotFoundError
from .default_places import DEFAULT_PLACES, T_PLACES


def find(name: str, *, places: T_PLACES = None, namespace: str = None, ns_only: bool = False) -> str:
    r"""
    find a config file with {name}

    :param name: name of the config file
    :param places: list of directories to search in
    :param namespace: namespace of the project
    :param ns_only: only search for {namespace}/{name}
    :return: filepath
    :raise ValueError: if no namespace is provided but ns_only is set
    :raise ConfigNotFoundError: config-file could not be found
    """
    if not namespace and ns_only:
        raise ValueError('ns_only can only be used if namespace is provided')

    if places is None:
        places = DEFAULT_PLACES

    for place in places:
        if not p.isdir(place):
            continue

        # check directly for the config file
        if not ns_only:
            fp = p.join(place, name)
            if p.isfile(fp):
                return fp

        if namespace:
            # check for the config file in the namespace folder
            fp = p.join(place, namespace, name)
            if p.isfile(fp):
                return fp

    # Error Message
    cfg = f"{namespace}/{name}" if ns_only else f"{name} or {namespace}/{name}"
    raise ConfigNotFoundError(f"{cfg} nowhere found")



#!/usr/bin/python3
# -*- coding=utf-8 -*-
r"""

"""
import typing as t
from ..exceptions import NotSupportedError
from .registry import REGISTRY, register_loader


def SUPPORTED_EXTENSIONS() -> t.Iterable[str]:
    r"""
    return currently supported file extensions

    eg: ini, conf, json
    """
    return tuple(REGISTRY.keys())


def SUPPORTED_LOADERS() -> t.Iterable[str]:
    r"""
    return the names of the currently supported loaders

    eg: json_loader, toml_loader
    """
    return tuple(
        loader.__name__ for loader in REGISTRY.values()
    )


def autoload(fp: str):
    r"""
    find the correct loader for the passed file, parses it and returns the result

    :param fp: path of the config-file
    :return:
    """
    import os.path as p
    ext = p.splitext(fp)[1][1:]
    if ext not in REGISTRY:
        raise NotSupportedError(f"files of type .{ext} are not supported")
    loader = REGISTRY[ext]
    return loader(fp)



#!/usr/bin/python3
# -*- coding=utf-8 -*-
r"""

"""
from ..exceptions import NotSupportedError
from .conf_loader import load_conf, ReturnType as ConfReturnType
from .xml_loader import load_xml, ReturnType as XmlReturnType
from .json_loader import load_json, ReturnType as JsonReturnType
from .jsonc_loader import load_jsonc, ReturnType as JsonCReturnType
try:
    from .json5_loader import load_json5, ReturnType as Json5ReturnType
except NotSupportedError:
    def load_json5(*_, **__):
        raise NotSupportedError()
    Json5ReturnType = None
try:
    from .toml_loader import load_toml, ReturnType as TomlReturnType
except NotSupportedError:
    def load_toml(*_, **__):
        raise NotSupportedError()
    TomlReturnType = None
try:
    from .yaml_loader import load_yaml, ReturnType as YamlReturnType
except NotSupportedError:
    def load_yaml(*_, **__):
        raise NotSupportedError()
    YamlReturnType = None



#!/usr/bin/python3
# -*- coding=utf-8 -*-
r"""

"""
import typing as t
import xml.etree.ElementTree
import xml.dom.minidom
from ..loader import register_loader


ReturnType: t.TypeAlias = xml.dom.minidom.Element


@register_loader('xml')
def load_xml(self) -> ReturnType:
    r"""
    <?xml version="1.0" encoding="UTF-8"?>
    <message>
        <warning>
             Hello World
        </warning>
    </message>
    """
    with open(self.fp, 'r') as file:
        document: xml.dom.minidom.Document = xml.dom.minidom.parse(file)
        # document.version, document.encoding, document.standalone
        root: xml.dom.minidom.Element = document.documentElement
    return root
